@@ -78,7 +78,7 @@ function CookieJarSelector() {
     } catch(e) {
         this.logger.log(4, "Cookie clearing exception: "+e);
     }
-  }
+  };
 
   // json would be a fine alternative to e4x, but is only available from
   // gecko1.9
@@ -118,7 +118,7 @@ function CookieJarSelector() {
             cookiesAsXml.appendChild(xml);
     }
     return cookiesAsXml;
-  }
+  };
 
   this._loadCookiesFromXml = function(cookiesAsXml) {
         if (typeof(cookiesAsXml) == "undefined" || !cookiesAsXml)
@@ -139,28 +139,159 @@ function CookieJarSelector() {
             var isSession = (xml.@isSession == 1);
             var isHttpOnly = (xml.@isHttpOnly == 1);
             //this.logger.log(2, "Loading cookie: "+host+":"+cname+" until: "+expiry);
-            try {
-                cookieManager.add(host, path, cname, value, isSecure, isSession,
-                        expiry);
-            } catch(e) {
-                // Api changed to add httpOnly cookies support. see mozilla bug #379408
-                if (e.result == Cr.NS_ERROR_XPC_NOT_ENOUGH_ARGS) {
-                    cookieManager.add(host, path, cname, value, isSecure, 
-                            isHttpOnly, isSession, expiry);
-                } 
-            }
+            cookieManager.add(host, path, cname, value, isSecure,
+                    isHttpOnly, isSession, expiry);
         }
-  }
+  };
 
   this._cookiesToFile = function(name) {
-      var file = getProfileFile("cookies-" + name + ".xml");
-      var foStream = Cc["@mozilla.org/network/file-output-stream;1"]
-            .createInstance(Ci.nsIFileOutputStream);
-      foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0); 
-      var data = this["cookiesobj-" + name].toString();
-      foStream.write(data, data.length);
-      foStream.close();
-  }
+    var file = getProfileFile("cookies-" + name + ".xml");
+    var foStream = Cc["@mozilla.org/network/file-output-stream;1"]
+          .createInstance(Ci.nsIFileOutputStream);
+    foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);
+    var data = this["cookiesobj-" + name].toString();
+    foStream.write(data, data.length);
+    foStream.close();
+  };
+
+  this._protectedCookiesToFile = function(name) {
+    var file = getProfileFile("protected-" + name + ".xml");
+    var foStream = Cc["@mozilla.org/network/file-output-stream;1"]
+        .createInstance(Ci.nsIFileOutputStream);
+    foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);
+    var data = this["protected-" + name].toString();
+    foStream.write(data, data.length);
+    foStream.close();
+  };
+
+  this.addProtectedCookie = function(cookie) {
+    var tor_enabled = this.prefs.getBoolPref("extensions.torbutton.tor_enabled");
+    var name = tor_enabled? "tor" : "nontor";
+    var cookies = this.getProtectedCookies(name);
+
+    if (typeof(cookies) == "undefined" || cookies == null
+            || cookies.toString() == "")
+      cookies = new XML('<cookies/>');
+    var xml = <cookie>{cookie.value}</cookie>;
+    xml.@name = cookie.name;
+    xml.@host = cookie.host;
+    xml.@path = cookie.path;
+    if (cookie.isSecure)
+      xml.@isSecure = 1;
+    if (cookie.isSession) {
+      xml.@isSession = 1;
+      // session cookies get fucked up expiry. Give it 1yr if
+      // the user wants to save their session cookies
+      xml.@expiry = Date.now()/1000 + 365*24*60*60;
+    } else {
+      xml.@expiry = cookie.expiry; 
+    }
+    if (cookie.isHttpOnly)
+      xml.@isHttpOnly = 1;
+
+    cookies.appendChild(xml);
+    this["protected-" + name] = cookies;
+
+    if (!this.prefs.getBoolPref("extensions.torbutton." + name + "_memory_jar")) {
+      // save protected cookies to file
+      this._protectedCookiesToFile(name);
+    } else {
+      try {
+        var file = getProfileFile("protected-" + name + ".xml");
+        if (file.exists()) {
+          file.remove(false);
+        }
+      } catch(e) {
+        this.logger.log(5, "Can't remove "+name+" cookie file: "+e);
+      }
+    }
+  };
+
+  this.getProtectedCookies = function(name) {
+      var file = getProfileFile("protected-" + name + ".xml");
+      if (!file.exists()) {
+        return this["protected-" + name];
+      }
+      var data = "";
+      var fstream = Cc["@mozilla.org/network/file-input-stream;1"]
+          .createInstance(Ci.nsIFileInputStream);
+      var sstream = Cc["@mozilla.org/scriptableinputstream;1"]
+          .createInstance(Ci.nsIScriptableInputStream);
+      fstream.init(file, -1, 0, 0);
+      sstream.init(fstream); 
+
+      var str = sstream.read(4096);
+      while (str.length > 0) {
+          data += str;
+          str = sstream.read(4096);
+      }
+
+      sstream.close();
+      fstream.close();
+      try {
+          var ret = XML(data);
+      } catch(e) { // file has been corrupted; XXX: handle error differently
+          this.logger.log(5, "Cookies corrupted: "+e);
+          try {
+              file.remove(false); //XXX: is it necessary to remove it ?
+              var ret = null;
+          } catch(e2) {
+              this.logger.log(5, "Can't remove file "+e);
+          }
+      }
+      return ret;
+  };
+
+  this.protectCookies = function(cookies) {
+    var tor_enabled = this.prefs.getBoolPref("extensions.torbutton.tor_enabled");
+    var name = tor_enabled? "tor" : "nontor";
+    this._writeProtectCookies(cookies,name);
+    if (!this.prefs.getBoolPref("extensions.torbutton." + name + "_memory_jar")) {
+      // save protected cookies to file
+      this._protectedCookiesToFile(name);
+    } else {
+      try {
+        var file = getProfileFile("protected-" + name + ".xml");
+        if (file.exists()) {
+          file.remove(false);
+        }
+      } catch(e) {
+        this.logger.log(5, "Can't remove "+name+" cookie file: "+e);
+      }
+    }
+  };
+
+  this._writeProtectCookies = function(cookies, name) {
+    var cookieManager =
+      Cc["@mozilla.org/cookiemanager;1"]
+      .getService(Ci.nsICookieManager);
+    var cookiesEnum = cookieManager.enumerator;
+    var cookiesAsXml = new XML('<cookies/>');
+    for (var i = 0; i < cookies.length; i++) {
+        var cookie = cookies[i];
+        var xml = <cookie>{cookie.value}</cookie>;
+        //this.logger.log(2, "Saving cookie: "+cookie.host+":"+cookie.name+" until: "+cookie.expiry);
+        xml.@name = cookie.name;
+        xml.@host = cookie.host;
+        xml.@path = cookie.path;
+        if (cookie.isSecure)
+            xml.@isSecure = 1;
+        if (cookie.isSession) {
+            xml.@isSession = 1;
+            // session cookies get fucked up expiry. Give it 1yr if
+            // the user wants to save their session cookies
+            xml.@expiry = Date.now()/1000 + 365*24*60*60;
+        } else {
+            xml.@expiry = cookie.expires; 
+        }
+        if (cookie.isHttpOnly)
+            xml.@isHttpOnly = 1;
+
+        // Save either session or non-session cookies this time around:
+        cookiesAsXml.appendChild(xml);
+    }
+    this["protected-" + name] = cookiesAsXml;
+  };
 
   this._cookiesFromFile = function(name) {
       var file = getProfileFile("cookies-" + name + ".xml");
@@ -194,7 +325,7 @@ function CookieJarSelector() {
           }
       }
       return ret;
-  }
+  };
 
   this.saveCookies = function(name) {
     // transition removes old tor-style cookie file
@@ -224,11 +355,59 @@ function CookieJarSelector() {
             this.logger.log(5, "Can't remove "+name+" cookie file "+e);
         }
     }
-    
+
     // ok, everything's fine
     this.logger.log(2, "Cookies saved");
   };
 
+  this.clearUnprotectedCookies = function(name) {
+    try {
+      var cookiesAsXml = this.getProtectedCookies(name);
+      if (cookiesAsXml == null || typeof(cookiesAsXml) == "undefined"
+              || cookiesAsXml.toString() == "") {
+        //file does not exist - no protected cookies. Clear them all.
+        this.clearCookies();
+        return;
+      }
+      var cookiemanager =
+        Cc["@mozilla.org/cookiemanager;1"]
+        .getService(Ci.nsICookieManager2);
+
+      var enumerator = cookiemanager.enumerator;
+      var count = 0;
+      var protcookie = false;
+
+      while (enumerator.hasMoreElements()) {
+        var nextCookie = enumerator.getNext();
+        if (!nextCookie) break;
+
+        nextCookie = nextCookie.QueryInterface(Components.interfaces.nsICookie);
+        for (var i = 0; i < cookiesAsXml.cookie.length(); i++) {
+          var xml = cookiesAsXml.cookie[i];
+          var cvalue = xml.toString();
+          var cname = xml.@name;
+          var chost = xml.@host;
+          var cpath = xml.@path;
+
+          protcookie = protcookie || (nextCookie.host == chost && nextCookie.name == cname && nextCookie.path == cpath);
+        }
+
+        if (!protcookie) {
+          cookiemanager.remove(nextCookie.host,
+                             nextCookie.name,
+                             nextCookie.path, false);
+        }
+        protcookie = false;
+      }
+      // Emit cookie-changed event. This instructs other components to clear their identifiers
+      // (Specifically DOM storage and safe browsing, but possibly others)
+      var obsSvc = Components.classes["@mozilla.org/observer-service;1"].getService(nsIObserverService);
+      obsSvc.notifyObservers(this, "cookie-changed", "cleared");
+    } catch (e) {
+      this.logger.log(3, "Error deleting unprotected cookies: " + e);
+    }
+  };
+
   this._oldLoadCookies = function(name, deleteSavedCookieJar) {
     var cookieManager =
       Cc["@mozilla.org/cookiemanager;1"]
@@ -286,7 +465,7 @@ function CookieJarSelector() {
     // load cookies from xml objects
     this._loadCookiesFromXml(this["cookiesobj-"+name]);
     this._loadCookiesFromXml(this["session-cookiesobj-"+name]);
-    
+
     // XXX: send a profile-do-change event?
 
     // ok, everything's fine
@@ -340,7 +519,7 @@ function CookieJarSelector() {
            jarThis.logger.log(3, "Neat. Timer fired during transition.");
            return;
        }
-       
+
        this.cookie_changed = false;
 
        if(tor_enabled) {
@@ -350,16 +529,10 @@ function CookieJarSelector() {
        }
        jarThis.logger.log(2, "Timer done. Cookies saved");
     }
-  }
+  };
 
 }
 
-/**
- * JS XPCOM component registration goop:
- *
- * Everything below is boring boilerplate and can probably be ignored.
- */
-
 const nsISupports = Components.interfaces.nsISupports;
 const nsIClassInfo = Components.interfaces.nsIClassInfo;
 const nsIObserver = Components.interfaces.nsIObserver;
@@ -386,7 +559,9 @@ CookieJarSelector.prototype =
   // make this an nsIClassInfo object
   flags: nsIClassInfo.DOM_OBJECT,
 
-  // method of nsIClassInfo
+  _xpcom_categories: [{category:"profile-after-change"}],
+  classID: kMODULE_CID,
+  contractID: kMODULE_CONTRACTID,
   classDescription: "CookieJarSelector",
 
   // method of nsIClassInfo
@@ -403,22 +578,22 @@ CookieJarSelector.prototype =
   observe : function(aSubject, aTopic, aData) {
        switch(aTopic) { 
         case "cookie-changed":
+            var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);          
             this.timerCallback.cookie_changed = true;
+    
+            if (aData == "added" && prefs.getBoolPref("extensions.torbutton.cookie_auto_protect") && ((!prefs.getBoolPref("extensions.torbutton.tor_memory_jar") && prefs.getBoolPref("extensions.torbutton.tor_enabled")) || (!prefs.getBoolPref("extensions.torbutton.nontor_memory_jar") && !prefs.getBoolPref("extensions.torbutton.tor_enabled"))))
+            {
+              this.addProtectedCookie(aSubject.QueryInterface(Components.interfaces.nsICookie2));//protect the new cookie!    
+            }
             break;
-        case "app-startup": 
+        case "profile-after-change":
             var obsSvc = Components.classes["@mozilla.org/observer-service;1"].getService(nsIObserverService);
-            obsSvc.addObserver(this, "profile-after-change", false); 
-            obsSvc.addObserver(this, "quit-application", false); 
-            obsSvc.addObserver(this, "cookie-changed", false); 
-            break;
-        case "profile-after-change": 
+            obsSvc.addObserver(this, "cookie-changed", false);
             // after profil loading, initialize a timer to call timerCallback
             // at a specified interval
             this.timer.initWithCallback(this.timerCallback, 60 * 1000, nsITimer.TYPE_REPEATING_SLACK); // 1 minute
+            this.logger.log(3, "Cookie jar selector got profile-after-change");
             break;
-        // put some stuff you want applied at firefox shutdown
-        case "quit-application":
-        break;
        }
   },
 
@@ -426,65 +601,12 @@ CookieJarSelector.prototype =
 
 }
 
-var CookieJarSelectorFactory = new Object();
-
-CookieJarSelectorFactory.createInstance = function (outer, iid)
-{
-  if (outer != null) {
-    Components.returnCode = Cr.NS_ERROR_NO_AGGREGATION;
-    return null;
-  }
-  if (!iid.equals(nsIClassInfo) &&
-      !iid.equals(nsIObserver) &&
-      !iid.equals(nsISupports)) {
-    Components.returnCode = Cr.NS_ERROR_NO_INTERFACE;
-    return null;
-  }
-  return new CookieJarSelector();
-}
-
-var CookieJarSelectorModule = new Object();
-
-CookieJarSelectorModule.registerSelf = 
-function (compMgr, fileSpec, location, type)
-{
-  compMgr = compMgr.QueryInterface(nsIComponentRegistrar);
-  compMgr.registerFactoryLocation(kMODULE_CID,
-                                  kMODULE_NAME,
-                                  kMODULE_CONTRACTID,
-                                  fileSpec, 
-                                  location, 
-                                  type);
-   var catman = Components.classes['@mozilla.org/categorymanager;1'].getService(nsICategoryManager);
-   catman.addCategoryEntry("app-startup", kMODULE_NAME, kMODULE_CONTRACTID, true, true);
-}
-
-CookieJarSelectorModule.unregisterSelf = 
-function (compMgr, fileSpec, location, type)
-{
-   var catman = Components.classes['@mozilla.org/categorymanager;1'].getService(nsICategoryManager);
-   catman.deleteCategoryEntry("app-startup", kMODULE_NAME, true);
-  compMgr = compMgr.QueryInterface(nsIComponentRegistrar);
-  compMgr.unregisterFactoryLocation(kMODULE_CID,
-                                  fileSpec);
-}
-
-CookieJarSelectorModule.getClassObject = function (compMgr, cid, iid)
-{
-  if (cid.equals(kMODULE_CID))
-    return CookieJarSelectorFactory;
-
-
-  Components.returnCode = Cr.NS_ERROR_NOT_REGISTERED;
-  return null;
-}
-
-CookieJarSelectorModule.canUnload = function (compMgr)
-{
-  return true;
-}
-
-function NSGetModule(compMgr, fileSpec)
-{
-  return CookieJarSelectorModule;
-}
+/**
+* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
+* XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
+*/
+Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+if (XPCOMUtils.generateNSGetFactory)
+    var NSGetFactory = XPCOMUtils.generateNSGetFactory([CookieJarSelector]);
+else
+    var NSGetModule = XPCOMUtils.generateNSGetModule([CookieJarSelector]);
