@@ -33,6 +33,8 @@ const Cr = Components.results;
 const Cc = Components.classes;
 const Ci = Components.interfaces;
 
+Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+
 // Retrieves the window object for a node or returns null if it isn't possible
 function getWindow(node) {
     if (node && node.nodeType != DNode.DOCUMENT_NODE)
@@ -257,6 +259,19 @@ ContentPolicy.prototype = {
                         }
                     }
 
+                    // Fix bug #2359: Firefox now loads dtd's from
+                    // resource urls with a valid host. We must
+                    // check for this and allow it.
+                    if (contentLocation.scheme == "resource") {
+                      try {
+                        targetHost = contentLocation.host;
+                      } catch(e) {
+                        this.logger.safe_log(3, "No resource host from: ",
+                                requestOrigin.spec + " for: " +
+                                contentLocation.spec);
+                      }
+                    }
+
                     if (("about:blank" == contentLocation.spec)) {
                         // ok, but don't return
                     } else if (("chrome" == targetScheme) && (targetHost in browserSources)) {
@@ -264,8 +279,9 @@ ContentPolicy.prototype = {
                                           requestOrigin.spec + " for: " +
                                           contentLocation.spec);
                         return ok;
-                    } else if ("file" == targetScheme) {
-                        // This fix is for bug 1014. XHTML documents need to source
+                    } else if ("file" == targetScheme ||
+                          ("resource" == targetScheme && targetHost == "gre")) {
+                        // This fix is for bugs 1014+2359. XHTML documents need to source
                         // a special dtd as a file url. The same origin policy should
                         // prevent other access to file urls, so this should be ok
                         // to just allow.
@@ -296,6 +312,11 @@ ContentPolicy.prototype = {
 			wind = node;
 		}
 
+        /*
+         * This clause does in fact occurr. It causes us to break hotmail
+         * by blocking some of their javascript..
+         *
+         * https://trac.torproject.org/projects/tor/ticket/3580
         if (contentType == 5) { // Object
             // Never seems to happen.. But it would be nice if we 
             // could handle it either here or shouldProcess, instead of in 
@@ -306,6 +327,7 @@ ContentPolicy.prototype = {
                 return block;
             }
         }
+        */
 
         if (!wind || !wind.top.location || !wind.top.location.href) {
             this.logger.safe_log(4, "Skipping no location: ",
@@ -440,94 +462,26 @@ ContentPolicy.prototype = {
     // API to add another addon to be protected from discovery
     addProtectedChromeHost: function(name) {
         protectedChromeHosts[name] = true;
-    }
+    },
+
+    _xpcom_categories: [{category:"content-policy"}],
+    classID: CSSB_CID,
+    contractID: CSSB_CONTRACTID,
+    classDescription: "Torbutton Content Policy",
+
+    // QueryInterface implementation, e.g. using the generateQI helper
+    QueryInterface: XPCOMUtils.generateQI(
+            [ Components.interfaces.nsIObserver,
+            Components.interfaces.nsISupports,
+            Components.interfaces.nsIContentPolicy ])
+
 };
 
-/*
- * Factory object
- */
-
-var ContentPolicyInstance = null;
-
-const factory = {
-	// nsIFactory interface implementation
-	createInstance: function(outer, iid) {
-		if (outer != null) {
-           Components.returnCode = Cr.NS_ERROR_NO_AGGREGATION;
-           return null;
-       }
-
-        if (!iid.equals(Components.interfaces.nsIContentPolicy) &&
-                !iid.equals(Components.interfaces.nsISupports)) {
-            Components.returnCode = Cr.NS_ERROR_NO_INTERFACE;          
-            return null;
-        }
-
-        if(!ContentPolicyInstance)
-            ContentPolicyInstance = new ContentPolicy();
-
-		return ContentPolicyInstance;
-	},
-
-	// nsISupports interface implementation
-	QueryInterface: function(iid) {
-		if (iid.equals(Components.interfaces.nsISupports) ||
-				iid.equals(Components.interfaces.nsIModule) ||
-				iid.equals(Components.interfaces.nsIFactory))
-			return this;
-
-        /*
-		if (!iid.equals(Components.interfaces.nsIClassInfo))
-			dump("CSS Blocker: factory.QI to an unknown interface: " + iid + "\n");
-        */
-
-        Components.returnCode = Cr.NS_ERROR_NO_INTERFACE;          
-        return null;   
-	}
-};
-
-
-/*
- * Module object
- */
-const module = {
-	registerSelf: function(compMgr, fileSpec, location, type) {
-		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
-		compMgr.registerFactoryLocation(CSSB_CID, 
-										"Torbutton content policy",
-										CSSB_CONTRACTID,
-										fileSpec, location, type);
-
-		var catman = Components.classes["@mozilla.org/categorymanager;1"]
-					 .getService(Components.interfaces.nsICategoryManager);
-		catman.addCategoryEntry("content-policy", CSSB_CONTRACTID,
-							CSSB_CONTRACTID, true, true);
-	},
-
-	unregisterSelf: function(compMgr, fileSpec, location) {
-		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
-
-		compMgr.unregisterFactoryLocation(CSSB_CID, fileSpec);
-		var catman = Components.classes["@mozilla.org/categorymanager;1"]
-					   .getService(Components.interfaces.nsICategoryManager);
-		catman.deleteCategoryEntry("content-policy", CSSB_CONTRACTID, true);
-	},
-
-	getClassObject: function(compMgr, cid, iid) {
-		if (cid.equals(CSSB_CID))
-            return factory;
-
-        Components.returnCode = Cr.NS_ERROR_NOT_REGISTERED;
-        return null;
-	},
-
-	canUnload: function(compMgr) {
-		return true;
-	}
-};
-
-function NSGetModule(comMgr, fileSpec) {
-	return module;
-}
-
-
+/**
+* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
+* XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
+*/
+if (XPCOMUtils.generateNSGetFactory)
+    var NSGetFactory = XPCOMUtils.generateNSGetFactory([ContentPolicy]);
+else
+    var NSGetModule = XPCOMUtils.generateNSGetModule([ContentPolicy]);
