@@ -11,16 +11,21 @@ const kMODULE_CONTRACTID_APP = "@mozilla.org/uriloader/external-helper-app-servi
 const kMODULE_CONTRACTID_PROTO = "@mozilla.org/uriloader/external-protocol-service;1";
 const kMODULE_CONTRACTID_MIME = "@mozilla.org/mime;1";
 
+const kMODULE_CONTRACTID_DRAG = "@mozilla.org/widget/dragservice;1";
+
+
 const kMODULE_CID = Components.ID("3da0269f-fc29-4e9e-a678-c3b1cafcf13f");
 
 /* Mozilla defined interfaces for FF3.0 */
 const kREAL_EXTERNAL_CID = "{A7F800E0-4306-11d4-98D0-001083010E9B}";
-
 const kExternalInterfaces = ["nsIObserver", "nsIMIMEService",
                              "nsIExternalHelperAppService",
                              "nsISupportsWeakReference", // XXX: Uh-oh...
                              "nsIExternalProtocolService",
                              "nsPIExternalAppLauncher"];
+                             
+const kREAL_DRAG_CID = "{8b5314bb-db01-11d2-96ce-0060b0fb9956}";
+const kDragInterfaces = ["nsIDragService"];
 
 const Cr = Components.results;
 const Cc = Components.classes;
@@ -38,6 +43,7 @@ function ExternalWrapper() {
   this.logger.log(3, "Component Load 0: New ExternalWrapper.");
 
   this._real_external = Components.classesByID[kREAL_EXTERNAL_CID];
+  this._real_drag = Components.classesByID[kREAL_DRAG_CID];
   this._interfaces = kExternalInterfaces;
 
   this._prefs = Components.classes["@mozilla.org/preferences-service;1"]
@@ -52,6 +58,15 @@ function ExternalWrapper() {
   };
     
   this.copyMethods(this._external());
+
+  this._drag = function() {
+    var drag = this._real_drag.getService();
+    for (var i = 0; i < kDragInterfaces.length; i++) {
+      drag.QueryInterface(Components.interfaces[kDragInterfaces]);
+    }
+    return drag;
+  };
+  this.copyMethods(this._drag());
 }
 
 ExternalWrapper.prototype =
@@ -62,8 +77,13 @@ ExternalWrapper.prototype =
       return this;
     }
 
-    var external = this._external().QueryInterface(iid);
-    this.copyMethods(external);
+    try {
+      var external = this._external().QueryInterface(iid);
+      this.copyMethods(external);
+    } catch(e) {
+      var drag = this._drag().QueryInterface(iid);
+      this.copyMethods(drag);
+    }
     return this;
   },
 
@@ -83,6 +103,10 @@ ExternalWrapper.prototype =
       interfaceList.push(Components.interfaces[this._interfaces[i]]);
     }
 
+    for (var i = 0; i < kDragInterfaces.length; i++) {
+      interfaceList.push(Components.interfaces[kDragInterfaces[i]]);
+    }
+
     count.value = interfaceList.length;
     return interfaceList;
   },
@@ -199,6 +223,34 @@ ExternalWrapper.prototype =
     return this._external().doContent(aMimeContentType, aRequest, aWindowContext, aForceSave);
   },
 
+  // from nsIDragService
+  invokeDragSessionWithImage: function(aDOMNode, aTransferableArray, aRegion, aActionType, aImage, aImageX, aImageY, aDragEvent, aDataTransfer) {
+    for(var i = 0; i < aTransferableArray.Count(); i++) {
+        this.logger.log(3, "Inspecting drag+drop transfer: "+i);
+        var tr = aTransferableArray.GetElementAt(i);
+        tr.QueryInterface(Ci.nsITransferable);
+
+        var flavors = tr.flavorsTransferableCanExport()
+                        .QueryInterface(Ci.nsISupportsArray);
+
+        for (var f=0; f < flavors.Count(); f++) {
+          var flavor =flavors.GetElementAt(f); 
+          flavor.QueryInterface(Ci.nsISupportsCString);
+
+          this.logger.log(3, "Got drag+drop flavor: "+flavor);
+          if (flavor == "text/x-moz-url" ||
+              flavor == "text/x-moz-url-data" ||
+              flavor == "text/uri-list" ||
+              flavor == "application/x-moz-file-promise-url") {
+            this.logger.log(3, "Removing "+flavor);
+            try { tr.removeDataFlavor(flavor); } catch(e) {}
+          }
+        }
+    }
+
+    return this._drag().invokeDragSessionWithImage(aDOMNode, aTransferableArray, aRegion, aActionType, aImage, aImageX, aImageY, aDragEvent, aDataTransfer);
+  },
+
 };
 
 var ExternalWrapperSingleton = null;
@@ -250,6 +302,14 @@ function (compMgr, fileSpec, location, type) {
                                   fileSpec,
                                   location,
                                   type);
+
+  compMgr.registerFactoryLocation(kMODULE_CID,
+                                  kMODULE_NAME,
+                                  kMODULE_CONTRACTID_DRAG,
+                                  fileSpec,
+                                  location,
+                                  type);
+
 };
 
 ExternalWrapperModule.getClassObject = function (compMgr, cid, iid)
